#include <stdlib.h>
#include <stdio.h>
#include <sys/shm.h>
#include <time.h>
#include <unistd.h>
#include "shared.h"

int GetNextFreeSlot(SharedStruct* sharedMem);
void StartProducer(SharedStruct* sharedMem);
void Produce(SharedStruct* sharedMem);
int produce_item(void);
int insert_item(int item, SharedStruct* sharedMem);


int main(int argc, char* argv[])
{
    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s SharedID# \n", argv[0]);
        exit(1);
    }
    srandom((unsigned int) time(NULL));
    int sharedID = StringToNumber(argv[1]);
    SharedStruct* sharedMem = shmat(sharedID, NULL, 0);
    StartProducer(sharedMem);
    return 0;
}

int GetNextFreeSlot(SharedStruct* sharedMem)
{
    int slot = -1;
    for (int i = 0; i < 10; i++)
    {
        if ((sharedMem->buffer[i] != -1) && (sharedMem->buffer[(i+1)%10] == -1))
        {
            slot = (i+1)%10;
            return slot;
        }
    }
    return slot;
}

//  Producer main function
//  TODO:   Add waiting for one producer at a time
//          Add ability to use shared memory
void StartProducer(SharedStruct* sharedMem)
{
    //  TODO: For testing purposes, revert max to 100
    for (int i = 0; i < 5; i++)
    {
        sem_wait(&sharedMem->empty);
        Produce(sharedMem);
        sem_post(&sharedMem->full);
    }
}

void Produce(SharedStruct* sharedMem)
{
    int insertNum;
    sem_wait(&sharedMem->mutex);
    insertNum = produce_item();
    insert_item(insertNum, sharedMem);
    printf("%d is inserted.\n", insertNum);
    sem_post(&sharedMem->mutex);
    return;
}

//  Generate a number between 0 and 9
int produce_item(void)
{
    return random() % 10;
}

//  Insert a number into the shared memory
int insert_item(int item, SharedStruct* sharedMem)
{
    // int slot = GetNextFreeSlot(sharedMem);
    // while (slot < 0)
    //     slot = GetNextFreeSlot(sharedMem);
    // sharedMem->buffer[slot] = item;
    sharedMem->buffer[0] = item;
    return 1;
}